package top.autopc.listen_call

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.IBinder
import android.util.Log
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import android.content.ComponentName
import android.telecom.TelecomManager
import android.telephony.TelephonyManager
import backgroundMessagePhoneCallMainChannel
import backgroundOnPhoneCallHandleArgumentKey
import backgroundPhoneCallHandleArgumentKey
import backgroundServiceSharedPreferencesName
import deviceNameArgumentKey
import getDeviceNameCallMethod
import initBackgroundCallMethod
import setDeviceNameCallMethod


private const val TAG="LCP";

/** ListenCallPlugin */
class ListenCallPlugin: FlutterPlugin, MethodCallHandler,ActivityAware {
  private lateinit var channel : MethodChannel
  private lateinit var activity : Activity
  private val serviceConnection=ServiceConnection()

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, backgroundMessagePhoneCallMainChannel)
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when(call.method){

      initBackgroundCallMethod->{
        val sharedPreferencesEditor=activity.getSharedPreferences(backgroundServiceSharedPreferencesName,Context.MODE_PRIVATE).edit();
        val backgroundPhoneCallHandle=call.argument<Long>(backgroundPhoneCallHandleArgumentKey)!!
        val backgroundOnPhoneCallHandle=call.argument<Long>(backgroundOnPhoneCallHandleArgumentKey)!!
        setSPCallbackHandle(sharedPreferencesEditor,backgroundPhoneCallHandle);
        setSPCallbackEventHandle(sharedPreferencesEditor,backgroundOnPhoneCallHandle);
        sharedPreferencesEditor.apply()
        activity.bindService(Intent(activity,PhoneStateListenerService::class.java),serviceConnection,Context.BIND_AUTO_CREATE);
        result.success(null)
      }
      setDeviceNameCallMethod->{
        val sharedPreferencesEditor=activity.getSharedPreferences(backgroundServiceSharedPreferencesName,Context.MODE_PRIVATE).edit();
        val deviceName=call.argument<String>(deviceNameArgumentKey)!!
        setSPDevicesName(sharedPreferencesEditor,deviceName)
        sharedPreferencesEditor.apply()
        result.success(null)
      }
      getDeviceNameCallMethod->{
        val sharedPreferences=activity.getSharedPreferences(backgroundServiceSharedPreferencesName,Context.MODE_PRIVATE)
        result.success(hashMapOf(deviceNameArgumentKey to sharedPreferences.getString(deviceNameArgumentKey,"")))
      }
      else->{
        result.notImplemented()
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivityForConfigChanges() {}

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

  override fun onDetachedFromActivity() {}

}

class ServiceConnection: ServiceConnection {

  override fun onServiceConnected(p0: ComponentName?, p1: IBinder?) {
    Log.d(TAG,"onServiceConnected");
  }

  override fun onServiceDisconnected(p0: ComponentName?) {
    Log.d(TAG,"onServiceDisconnected");
  }

}