package top.autopc.listen_call

import spCallbackHandleName
import spCallbackHandleEventName
import android.app.Service
import android.content.Context
import android.content.Intent
import android.os.Binder
import android.os.IBinder
import android.telephony.PhoneStateListener
import android.telephony.TelephonyManager
import android.util.Log
import backgroundMessagePhoneCallServiceChannel
import backgroundServiceSharedPreferencesName
import io.flutter.FlutterInjector
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.embedding.engine.FlutterJNI
import io.flutter.embedding.engine.dart.DartExecutor
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.view.FlutterCallbackInformation

private const val TAG = "PSLS";

class PhoneStateListenerService:  Service(),MethodChannel.MethodCallHandler {
    private lateinit var customPhoneStateListener:CallPhoneListener
    private lateinit var backgroundChannel:MethodChannel
    private lateinit var flutterEngine:FlutterEngine
    private var backgroundOnPhoneCallHandle:Long = 0
    private val mBinder: IBinder = PhoneStateListenerServiceBinder(this)

    override fun onBind(intent: Intent?): IBinder {
        Log.d(TAG,"on bind")
        initService(applicationContext)
        val telephonyManagerService: TelephonyManager? = applicationContext.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager?
        customPhoneStateListener = CallPhoneListener(applicationContext,backgroundChannel)
        telephonyManagerService?.listen(customPhoneStateListener,PhoneStateListener.LISTEN_CALL_STATE)
        if (telephonyManagerService==null) {
            Log.d(TAG,"telephonyManagerService init fail")
        }
        else {
            Log.d(TAG,"telephonyManagerService init success")
        }
        return mBinder
    }

    /**
     * init flutter loader
     */
    private fun initService(context:Context){
        val preferences = getSharedPreferences(backgroundServiceSharedPreferencesName,Context.MODE_PRIVATE)
        val backgroundPhoneCallHandle=preferences.getLong(spCallbackHandleName,0)
        backgroundOnPhoneCallHandle=preferences.getLong(spCallbackHandleEventName,0)

        val flutterLoader = FlutterInjector.instance().flutterLoader().apply {
            startInitialization(context)
            ensureInitializationComplete(context, null)
        }

        val dartEntryPoint = DartExecutor.DartCallback(
                context.assets,
                flutterLoader.findAppBundlePath(),
                FlutterCallbackInformation.lookupCallbackInformation(backgroundPhoneCallHandle)
        )

        flutterEngine = FlutterEngine(context, flutterLoader, FlutterJNI())
        Log.d(TAG,dartEntryPoint.toString());
        flutterEngine.dartExecutor.executeDartCallback(dartEntryPoint)
        backgroundChannel = MethodChannel(flutterEngine.dartExecutor,backgroundMessagePhoneCallServiceChannel)
        backgroundChannel.setMethodCallHandler(this)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        Log.d(TAG,arrayOf(call.method,call.arguments.toString()).joinToString(", "))
    }
}

class PhoneStateListenerServiceBinder(private val service: PhoneStateListenerService): Binder() {
    fun getService(): PhoneStateListenerService {
        return service
    }
}