import 'dart:async';
import 'dart:ui';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_easy_permission/easy_permissions.dart';
import 'package:listen_call/protocol.dart';

const spCallbackHandleEventName = "SPCallbackHandleEventName";

typedef FutureOr<void> OnPhoneCall(Map<String, dynamic> callPhoneInfo);

class ListenCall {
  static const MethodChannel _channel = MethodChannel(backgroundMessagePhoneCallMainChannel);

  static Future initBackgroundCall(OnPhoneCall onPhoneCall) {
    return Future(() async {
      final backgroundPhoneCallHandle = PluginUtilities.getCallbackHandle(backgroundPhoneCall)!.toRawHandle();
      final onPhoneCallHandle = PluginUtilities.getCallbackHandle(onPhoneCall)!.toRawHandle();
      _channel.invokeMapMethod(initBackgroundCallMethod, {
        backgroundPhoneCallHandleArgumentKey: backgroundPhoneCallHandle,
        backgroundOnPhoneCallHandleArgumentKey: onPhoneCallHandle,
      });
    });
  }

  static Future setDeviceName(String deviceName) async {
    return await _channel.invokeMapMethod(setDeviceNameCallMethod, {deviceNameArgumentKey: deviceName});
  }

  static Future<String> getDeviceName() async {
    return (await _channel.invokeMapMethod(getDeviceNameCallMethod))?["deviceNameArgumentKey"] ?? "";
  }

  static Future get requestPhonePermission async {
    FlutterEasyPermission.request(
      perms: [
        Permissions.READ_PHONE_STATE,
        Permissions.CALL_PHONE,
        Permissions.PROCESS_OUTGOING_CALLS,
        Permissions.READ_CALL_LOG,
      ],
      permsGroup: [],
    );
  }
}

abstract class ListenCallStateCode {
  static const closeCallPhone = 0;
  static const waitCallPhone = 1;
  static const openCallPhone = 2;
}

void backgroundPhoneCall() {
  WidgetsFlutterBinding.ensureInitialized();
  const _backgroundChannel = MethodChannel(backgroundMessagePhoneCallServiceChannel);
  _backgroundChannel.setMethodCallHandler((call) async {
    assert((() {
      // ignore: avoid_print
      print("callback $call!!!");
      return true;
    })());

    switch (call.method) {
      case onCallPhoneCallMethod:
        // 忽略掉无用请求
        if ((call.arguments["incomingNumber"] as String).isEmpty && call.arguments["state"] == 0) {
          break;
        }

        final f = PluginUtilities.getCallbackFromHandle(CallbackHandle.fromRawHandle(call.arguments["callback"]));

        /// [f] 非空
        if (f != null) {
          f(Map<String, dynamic>.from(call.arguments));
        } else {
          assert((() {
            // ignore: avoid_print
            print("callback is null !!!");
            return true;
          })());
        }
        break;
      default:
        assert((() {
          // ignore: avoid_print
          print([call.method, "not found", "arguments", call.arguments]);
          return true;
        })());
    }
  });
}
